"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllProductVariantIds = exports.getAllProductIds = exports.productExists = exports.createProduct = exports.getProductIdsFromVariantIds = exports.getProductIdFromVariantId = exports.bulkUpdateProductsTags = exports.updateProductTagsMutation = void 0;
const api_1 = require("./api");
const bulk_operations_1 = require("./bulk-operations");
const promises_1 = require("timers/promises");
const promises_2 = require("node:stream/promises");
exports.updateProductTagsMutation = (0, api_1.gql) `
  mutation productUpdate($input: ProductInput!) {
    productUpdate(input: $input) {
      product {
        id
        tags
      }
      userErrors {
        field
        message
      }
    }
  }
`;
async function bulkUpdateProductsTags(JSONLReadStream) {
    return (0, bulk_operations_1.bulkUpdate)({ JSONLReadStream, mutation: exports.updateProductTagsMutation });
}
exports.bulkUpdateProductsTags = bulkUpdateProductsTags;
async function getProductIdFromVariantId(variantId) {
    try {
        const res = await api_1.graphqlClient.request(`
        {
          productVariant(id: "gid://shopify/ProductVariant/${variantId}") {
            product {
              id
            }
          }
        }
      `);
        const map = {};
        map[variantId] = res.productVariant.product.id;
        return map;
    }
    catch (e) {
        throw e;
    }
}
exports.getProductIdFromVariantId = getProductIdFromVariantId;
async function getProductIdsFromVariantIds(variantIds, limit = 475, wait = 20000) {
    try {
        const reqInput = [...variantIds];
        const results = {};
        let failuresCounts = 0;
        while (reqInput.length) {
            const tempInput = [];
            while (tempInput.length < limit + 1 && reqInput.length) {
                const val = reqInput.pop();
                if (val)
                    tempInput.push(val);
            }
            const createQuery = (id) => `
          _${id}: productVariant(id: "gid://shopify/ProductVariant/${id}") {
            product {
              id
            }
          }
        `;
            const query = `
          {
            ${tempInput.map((id) => createQuery(id)).join(" ")}
          }
        `;
            const res2 = await api_1.graphqlClient.request(query).catch((e) => {
                failuresCounts++;
                console.error(e);
            });
            if (res2) {
                for (const [vId, value] of Object.entries(res2)) {
                    if (vId && value) {
                        results[vId.replace("_", "")] = value.product.id;
                    }
                    else {
                        console.log(vId, value);
                    }
                }
            }
            if (wait === 0)
                continue;
            await (0, promises_1.setTimeout)(wait);
        }
        console.log({ failuresCounts });
        return results;
    }
    catch (e) {
        throw e;
    }
}
exports.getProductIdsFromVariantIds = getProductIdsFromVariantIds;
async function createProduct(input) {
    const mutation = (0, api_1.gql) `
    mutation productCreate($input: ProductInput!) {
      productCreate(input: $input) {
        product {
          id
        }
        userErrors {
          field
          message
        }
      }
    }
  `;
    return await api_1.graphqlClient.request(mutation, input);
}
exports.createProduct = createProduct;
async function productExists(sku) {
    const query = (0, api_1.gql) `
  {
    products(first:1,query:"sku:${sku}") {
      nodes {
        id
      }
    }
  }
  `;
    const res = await api_1.graphqlClient.request(query);
    if (res.products.nodes?.length) {
        return { exists: true, productId: res.products.nodes[0].id };
    }
    return false;
}
exports.productExists = productExists;
async function getAllProductIds(writeStream) {
    const query = `
  query {
    products {
      edges {
        node {
          id
          variants(first:20){
            edges{
              node{
                id
                sku
              }
            }
          }
        }
      }
    }
  }
  `;
    const readStream = await (0, bulk_operations_1.bulkQueryComplete)({ query });
    await (0, promises_2.pipeline)(readStream, writeStream);
}
exports.getAllProductIds = getAllProductIds;
async function getAllProductVariantIds(writeStream) {
    const query = `
  query {
    productVariants {
      edges {
        node {
          id
          sku
          product {
            id
          }
        }
      }
    }
  }
  `;
    const readStream = await (0, bulk_operations_1.bulkQueryComplete)({ query });
    await (0, promises_2.pipeline)(readStream, writeStream);
}
exports.getAllProductVariantIds = getAllProductVariantIds;
